@@ -1810,6 +1810,11 @@ pub(crate) fn emit(
             // Nothing.
         }
 
+        Inst::LabelAddress { dst, label } => {
+            // Emit an LEA with a LabelUse given this label.
+            asm::inst::leaq_rm::new(*dst, Amode::rip_relative(*label)).emit(sink, info, state);
+        }
+
         Inst::External { inst } => {
             let frame = state.frame_layout();
             emit_maybe_shrink(
