@@ -349,6 +349,11 @@ impl fmt::Debug for Slice<Path> {
     /// If alternate format is specified, e.g. `{:#?}`, the slice is assumed to
     /// be conventionally UTF-8 and converted to a [`String`] lossily with
     /// [`Path::to_string_lossy`].
+    // Suppress due to false positives in the lint:
+    //
+    // - https://github.com/rust-lang/rust-clippy/issues/14952
+    // - https://github.com/rust-lang/rust-clippy/issues/14977
+    #[allow(clippy::unnecessary_debug_formatting)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         if f.alternate() {
             write!(f, "{:?}", self.as_slice().to_string_lossy())
@@ -555,30 +560,22 @@ mod tests {
     #[test]
     #[cfg(feature = "cstr")]
     fn test_interned_static_cstr_debug_format() {
-        let s = Interned::from(Cow::Borrowed(
-            CStr::from_bytes_with_nul(b"abc\x00").unwrap(),
-        ));
+        let s = Interned::from(Cow::Borrowed(c"abc"));
         let mut buf = String::new();
         write!(&mut buf, "{s:?}").unwrap();
         assert_eq!(buf, "\"abc\"");
 
-        let s = Interned::from(Cow::Borrowed(
-            CStr::from_bytes_with_nul(b"\xFF\x00").unwrap(),
-        ));
+        let s = Interned::from(Cow::Borrowed(c"\xFF"));
         let mut buf = String::new();
         write!(&mut buf, "{s:?}").unwrap();
         assert_eq!(buf, r#""\xff""#);
 
-        let s = Interned::from(Cow::Borrowed(
-            CStr::from_bytes_with_nul(b"abc\x00").unwrap(),
-        ));
+        let s = Interned::from(Cow::Borrowed(c"abc"));
         let mut buf = String::new();
         write!(&mut buf, "{s:#?}").unwrap();
         assert_eq!(buf, "\"abc\"");
 
-        let s = Interned::from(Cow::Borrowed(
-            CStr::from_bytes_with_nul(b"\xFF\x00").unwrap(),
-        ));
+        let s = Interned::from(Cow::Borrowed(c"\xFF"));
         let mut buf = String::new();
         write!(&mut buf, "{s:#?}").unwrap();
         assert_eq!(buf, "\"\u{FFFD}\"");
@@ -587,30 +584,22 @@ mod tests {
     #[test]
     #[cfg(feature = "cstr")]
     fn test_interned_owned_cstring_debug_format() {
-        let s = Interned::<CStr>::from(Cow::Owned(
-            CStr::from_bytes_with_nul(b"abc\x00").unwrap().to_owned(),
-        ));
+        let s = Interned::<CStr>::from(Cow::Owned(c"abc".to_owned()));
         let mut buf = String::new();
         write!(&mut buf, "{s:?}").unwrap();
         assert_eq!(buf, "\"abc\"");
 
-        let s = Interned::<CStr>::from(Cow::Owned(
-            CStr::from_bytes_with_nul(b"\xFF\x00").unwrap().to_owned(),
-        ));
+        let s = Interned::<CStr>::from(Cow::Owned(c"\xFF".to_owned()));
         let mut buf = String::new();
         write!(&mut buf, "{s:?}").unwrap();
         assert_eq!(buf, r#""\xff""#);
 
-        let s = Interned::<CStr>::from(Cow::Owned(
-            CStr::from_bytes_with_nul(b"abc\x00").unwrap().to_owned(),
-        ));
+        let s = Interned::<CStr>::from(Cow::Owned(c"abc".to_owned()));
         let mut buf = String::new();
         write!(&mut buf, "{s:#?}").unwrap();
         assert_eq!(buf, "\"abc\"");
 
-        let s = Interned::<CStr>::from(Cow::Owned(
-            CStr::from_bytes_with_nul(b"\xFF\x00").unwrap().to_owned(),
-        ));
+        let s = Interned::<CStr>::from(Cow::Owned(c"\xFF".to_owned()));
         let mut buf = String::new();
         write!(&mut buf, "{s:#?}").unwrap();
         assert_eq!(buf, "\"\u{FFFD}\"");
