@@ -13,6 +13,7 @@ mod locals;
 mod memories;
 mod producers;
 mod tables;
+mod tags;
 mod types;
 
 use crate::emit::{Emit, EmitContext, IdsToIndices};
@@ -36,6 +37,7 @@ pub use crate::module::locals::ModuleLocals;
 pub use crate::module::memories::{Memory, MemoryId, ModuleMemories};
 pub use crate::module::producers::ModuleProducers;
 pub use crate::module::tables::{ModuleTables, Table, TableId};
+pub use crate::module::tags::{ModuleTags, Tag, TagId, TagKind};
 pub use crate::module::types::ModuleTypes;
 use crate::parse::IndicesToIds;
 use anyhow::{bail, Context};
@@ -61,6 +63,8 @@ pub struct Module {
     pub locals: ModuleLocals,
     pub exports: ModuleExports,
     pub memories: ModuleMemories,
+    /// Tags for exception handling
+    pub tags: ModuleTags,
     /// Registration of passive data segments, if any
     pub data: ModuleData,
     /// Registration of passive element segments, if any
@@ -225,8 +229,8 @@ impl Module {
                     validator.data_count_section(count, &range)?;
                     ret.reserve_data(count, &mut indices);
                 }
-                Payload::CodeSectionStart { count, range, .. } => {
-                    validator.code_section_start(count, &range)?;
+                Payload::CodeSectionStart { range, .. } => {
+                    validator.code_section_start(&range)?;
                     ret.funcs.code_section_offset = range.start;
                 }
                 Payload::CodeSectionEntry(body) => {
@@ -236,35 +240,37 @@ impl Module {
                     local_functions.push((body, validator));
                 }
                 Payload::CustomSection(s) => {
-                    let result =
-                        match s.name() {
-                            "producers" => wasmparser::ProducersSectionReader::new(
-                                BinaryReader::new(s.data(), s.data_offset(), wasm_features),
-                            )
-                            .map_err(anyhow::Error::from)
-                            .and_then(|s| ret.parse_producers_section(s)),
-                            "name" => {
-                                let name_section_reader = wasmparser::NameSectionReader::new(
-                                    BinaryReader::new(s.data(), s.data_offset(), wasm_features),
-                                );
-                                ret.parse_name_section(name_section_reader, &indices)
+                    let result = match s.name() {
+                        "producers" => wasmparser::ProducersSectionReader::new(
+                            BinaryReader::new_features(s.data(), s.data_offset(), wasm_features),
+                        )
+                        .map_err(anyhow::Error::from)
+                        .and_then(|s| ret.parse_producers_section(s)),
+                        "name" => {
+                            let name_section_reader =
+                                wasmparser::NameSectionReader::new(BinaryReader::new_features(
+                                    s.data(),
+                                    s.data_offset(),
+                                    wasm_features,
+                                ));
+                            ret.parse_name_section(name_section_reader, &indices)
+                        }
+                        name => {
+                            log::debug!("parsing custom section `{}`", name);
+                            if name.starts_with(".debug") {
+                                debug_sections.push(RawCustomSection {
+                                    name: name.to_string(),
+                                    data: s.data().to_vec(),
+                                });
+                            } else {
+                                ret.customs.add(RawCustomSection {
+                                    name: name.to_string(),
+                                    data: s.data().to_vec(),
+                                });
                             }
-                            name => {
-                                log::debug!("parsing custom section `{}`", name);
-                                if name.starts_with(".debug") {
-                                    debug_sections.push(RawCustomSection {
-                                        name: name.to_string(),
-                                        data: s.data().to_vec(),
-                                    });
-                                } else {
-                                    ret.customs.add(RawCustomSection {
-                                        name: name.to_string(),
-                                        data: s.data().to_vec(),
-                                    });
-                                }
-                                continue;
-                            }
-                        };
+                            continue;
+                        }
+                    };
                     if let Err(e) = result {
                         log::warn!("failed to parse `{}` custom section {}", s.name(), e);
                     }
@@ -279,24 +285,15 @@ impl Module {
                     continue;
                 }
 
-                // component module proposal is not implemented yet.
-                Payload::ModuleSection { .. }
-                | Payload::InstanceSection(..)
-                | Payload::CoreTypeSection(..)
-                | Payload::ComponentSection { .. }
-                | Payload::ComponentInstanceSection(..)
-                | Payload::ComponentAliasSection(..)
-                | Payload::ComponentTypeSection(..)
-                | Payload::ComponentCanonicalSection(..)
-                | Payload::ComponentStartSection { .. }
-                | Payload::ComponentImportSection(..)
-                | Payload::ComponentExportSection(..) => {
-                    bail!("not supported yet");
-                }
-
-                // exception handling is not implemented yet.
+                // Parse exception handling tags
                 Payload::TagSection(s) => {
                     validator.tag_section(&s)?;
+                    ret.parse_tags(s, &mut indices)?;
+                }
+
+                // Among other things, the component module proposal is not
+                // implemented yet.
+                _ => {
                     bail!("not supported yet");
                 }
             }
@@ -353,7 +350,7 @@ impl Module {
         self.funcs.emit_func_section(&mut cx);
         self.tables.emit(&mut cx);
         self.memories.emit(&mut cx);
-        // TODO: tag section
+        self.tags.emit(&mut cx);
         self.globals.emit(&mut cx);
         self.exports.emit(&mut cx);
         if let Some(start) = self.start {
@@ -497,6 +494,15 @@ impl Module {
                         }
                     }
                 }
+                wasmparser::Name::Tag(names) => {
+                    for name in names {
+                        let naming = name?;
+                        match indices.get_tag(naming.index) {
+                            Ok(id) => self.tags.get_mut(id).name = Some(naming.name.to_string()),
+                            Err(e) => warn!("in name section: {}", e),
+                        }
+                    }
+                }
                 wasmparser::Name::Local(l) => {
                     for f in l {
                         let f = f?;
@@ -527,7 +533,6 @@ impl Module {
                 wasmparser::Name::Unknown { ty, .. } => warn!("unknown name subsection {}", ty),
                 wasmparser::Name::Label(_) => warn!("labels name subsection ignored"),
                 wasmparser::Name::Field(_) => warn!("fields name subsection ignored"),
-                wasmparser::Name::Tag(_) => warn!("tags name subsection ignored"),
             }
         }
         Ok(())
@@ -625,6 +630,15 @@ fn emit_name_section(cx: &mut EmitContext) {
         .collect::<Vec<_>>();
     data.sort_by_key(|p| p.0); // sort by index
 
+    let mut tags = cx
+        .module
+        .tags
+        .iter()
+        .filter_map(|tag| tag.name.as_ref().map(|name| (tag, name)))
+        .map(|(tag, name)| (cx.indices.get_tag_index(tag.id()), name))
+        .collect::<Vec<_>>();
+    tags.sort_by_key(|p| p.0); // sort by index
+
     if cx.module.name.is_none()
         && funcs.is_empty()
         && locals.is_empty()
@@ -634,6 +648,7 @@ fn emit_name_section(cx: &mut EmitContext) {
         && globals.is_empty()
         && elements.is_empty()
         && data.is_empty()
+        && tags.is_empty()
     {
         return;
     }
@@ -698,6 +713,14 @@ fn emit_name_section(cx: &mut EmitContext) {
         wasm_name_section.globals(&name_map);
     }
 
+    if !tags.is_empty() {
+        let mut name_map = wasm_encoder::NameMap::new();
+        for (index, name) in tags {
+            name_map.append(index, name);
+        }
+        wasm_name_section.tags(&name_map);
+    }
+
     if !elements.is_empty() {
         let mut name_map = wasm_encoder::NameMap::new();
         for (index, name) in elements {
