@@ -568,6 +568,12 @@ fn create_visit(variants: &[WalrusVariant]) -> impl quote::ToTokens {
                 // ...
             }
 
+            /// Visit `TagId`
+            #[inline]
+            fn visit_tag_id(&mut self, tag: &crate::TagId) {
+                // ...
+            }
+
             /// Visit `Value`.
             #[inline]
             fn visit_value(&mut self, value: &crate::ir::Value) {
@@ -658,6 +664,12 @@ fn create_visit(variants: &[WalrusVariant]) -> impl quote::ToTokens {
                 // ...
             }
 
+            /// Visit `TagId`
+            #[inline]
+            fn visit_tag_id_mut(&mut self, tag: &mut crate::TagId) {
+                // ...
+            }
+
             /// Visit `Value`.
             #[inline]
             fn visit_value_mut(&mut self, value: &mut crate::ir::Value) {
