@@ -7,18 +7,18 @@ pub(crate) use canonical::InternRecGroup;
 use self::arc::MaybeOwned;
 use super::{
     check_max, combine_type_sizes,
-    operators::{ty_to_str, OperatorValidator, OperatorValidatorAllocations},
+    operators::{OperatorValidator, OperatorValidatorAllocations, ty_to_str},
     types::{CoreTypeId, EntityType, RecGroupId, TypeAlloc, TypeList},
 };
 #[cfg(feature = "simd")]
 use crate::VisitSimdOperator;
 use crate::{
-    limits::*, BinaryReaderError, ConstExpr, Data, DataKind, Element, ElementKind, ExternalKind,
-    FuncType, Global, GlobalType, HeapType, MemoryType, RecGroup, RefType, Result, SubType, Table,
-    TableInit, TableType, TagType, TypeRef, UnpackedIndex, ValType, VisitOperator, WasmFeatures,
-    WasmModuleResources,
+    BinaryReaderError, ConstExpr, Data, DataKind, Element, ElementKind, ExternalKind, FrameKind,
+    FrameStack, FuncType, Global, GlobalType, HeapType, MemoryType, RecGroup, RefType, Result,
+    SubType, Table, TableInit, TableType, TagType, TypeRef, UnpackedIndex, ValType, VisitOperator,
+    WasmFeatures, WasmModuleResources, limits::*,
 };
-use crate::{prelude::*, CompositeInnerType};
+use crate::{CompositeInnerType, prelude::*};
 use alloc::sync::Arc;
 use core::mem;
 
@@ -217,7 +217,7 @@ impl ModuleState {
             },
         };
 
-        let mut ops = expr.get_operators_reader();
+        let mut ops = expr.get_binary_reader();
         while !ops.eof() {
             validator.offset = ops.original_position();
             ops.visit_operator(&mut validator)??;
@@ -466,6 +466,12 @@ impl ModuleState {
         impl<'a> VisitSimdOperator<'a> for VisitConstOperator<'a> {
             crate::for_each_visit_simd_operator!(define_visit_operator);
         }
+
+        impl<'a> FrameStack for VisitConstOperator<'a> {
+            fn current_frame(&self) -> Option<FrameKind> {
+                Some(self.ops.get_frame(0)?.kind)
+            }
+        }
     }
 }
 
@@ -742,7 +748,10 @@ impl Module {
 
         let page_size = if let Some(page_size_log2) = ty.page_size_log2 {
             if !self.features().custom_page_sizes() {
-                return Err(BinaryReaderError::new("the custom page sizes proposal must be enabled to customize a memory's page size", offset));
+                return Err(BinaryReaderError::new(
+                    "the custom page sizes proposal must be enabled to customize a memory's page size",
+                    offset,
+                ));
             }
             // Currently 2**0 and 2**16 are the only valid page sizes, but this
             // may be relaxed to allow any power of two in the future.
@@ -1174,13 +1183,11 @@ impl WasmModuleResources for ValidatorResources {
 }
 
 const _: () = {
-    fn assert_send<T: Send>() {}
+    const fn assert_send<T: Send>() {}
 
     // Assert that `ValidatorResources` is Send so function validation
     // can be parallelizable
-    fn assert() {
-        assert_send::<ValidatorResources>();
-    }
+    assert_send::<ValidatorResources>();
 };
 
 mod arc {
