@@ -281,7 +281,7 @@ impl Func {
         #[cfg(feature = "component-model-async")]
         {
             self.instance
-                .run_concurrent(&mut store, async |store| {
+                .run_concurrent_trap_on_idle(&mut store, async |store| {
                     self.call_concurrent_dynamic(store, params, results, false)
                         .await
                         .map(drop)
@@ -734,6 +734,10 @@ impl Func {
             // panic, even if the function call below traps.
             flags.set_needs_post_return(false);
 
+            // Post return functions are forbidden from calling imports or
+            // intrinsics.
+            flags.set_may_leave(false);
+
             // If the function actually had a `post-return` configured in its
             // canonical options that's executed here.
             //
@@ -750,9 +754,10 @@ impl Func {
             }
 
             // And finally if everything completed successfully then the "may
-            // enter" flag is set to `true` again here which enables further use
-            // of the component.
+            // enter" and "may leave" flags are set to `true` again here which
+            // enables further use of the component.
             flags.set_may_enter(true);
+            flags.set_may_leave(true);
 
             let (calls, host_table, _, instance) = store
                 .0
