@@ -378,9 +378,11 @@ impl Instance {
     pub(crate) fn resource_new32(
         self,
         store: &mut StoreOpaque,
+        caller: RuntimeComponentInstanceIndex,
         ty: TypeResourceTableIndex,
         rep: u32,
     ) -> Result<u32> {
+        self.id().get(store).check_may_leave(caller)?;
         let (calls, _, _, instance) = store.component_resource_state_with_instance(self);
         resource_tables(calls, instance).resource_new(TypedResource::Component { ty, rep })
     }
@@ -390,9 +392,11 @@ impl Instance {
     pub(crate) fn resource_rep32(
         self,
         store: &mut StoreOpaque,
+        caller: RuntimeComponentInstanceIndex,
         ty: TypeResourceTableIndex,
         index: u32,
     ) -> Result<u32> {
+        self.id().get(store).check_may_leave(caller)?;
         let (calls, _, _, instance) = store.component_resource_state_with_instance(self);
         resource_tables(calls, instance).resource_rep(TypedResourceIndex::Component { ty, index })
     }
@@ -401,9 +405,11 @@ impl Instance {
     pub(crate) fn resource_drop(
         self,
         store: &mut StoreOpaque,
+        caller: RuntimeComponentInstanceIndex,
         ty: TypeResourceTableIndex,
         index: u32,
     ) -> Result<Option<u32>> {
+        self.id().get(store).check_may_leave(caller)?;
         let (calls, _, _, instance) = store.component_resource_state_with_instance(self);
         resource_tables(calls, instance).resource_drop(TypedResourceIndex::Component { ty, index })
     }
