@@ -579,33 +579,42 @@ fn inflate_latin1_bytes(dst: &mut [u16], latin1_bytes_so_far: usize) -> &mut [u1
 fn resource_new32(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     resource: u32,
     rep: u32,
 ) -> Result<u32> {
+    let caller_instance = RuntimeComponentInstanceIndex::from_u32(caller_instance);
     let resource = TypeResourceTableIndex::from_u32(resource);
-    instance.resource_new32(store, resource, rep)
+    instance.resource_new32(store, caller_instance, resource, rep)
 }
 
 fn resource_rep32(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     resource: u32,
     idx: u32,
 ) -> Result<u32> {
+    let caller_instance = RuntimeComponentInstanceIndex::from_u32(caller_instance);
     let resource = TypeResourceTableIndex::from_u32(resource);
-    instance.resource_rep32(store, resource, idx)
+    instance.resource_rep32(store, caller_instance, resource, idx)
 }
 
 fn resource_drop(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     resource: u32,
     idx: u32,
 ) -> Result<ResourceDropRet> {
+    let caller_instance = RuntimeComponentInstanceIndex::from_u32(caller_instance);
     let resource = TypeResourceTableIndex::from_u32(resource);
-    Ok(ResourceDropRet(
-        instance.resource_drop(store, resource, idx)?,
-    ))
+    Ok(ResourceDropRet(instance.resource_drop(
+        store,
+        caller_instance,
+        resource,
+        idx,
+    )?))
 }
 
 struct ResourceDropRet(Option<u32>);
@@ -693,6 +702,7 @@ fn backpressure_modify(
 unsafe fn task_return(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     storage: *mut u8,
@@ -700,6 +710,7 @@ unsafe fn task_return(
 ) -> Result<()> {
     instance.task_return(
         store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeTupleIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         unsafe { core::slice::from_raw_parts(storage.cast(), storage_len) },
@@ -730,22 +741,36 @@ fn waitable_set_new(
 fn waitable_set_wait(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller: u32,
     options: u32,
     set: u32,
     payload: u32,
 ) -> Result<u32> {
-    instance.waitable_set_wait(store, OptionsIndex::from_u32(options), set, payload)
+    instance.waitable_set_wait(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller),
+        OptionsIndex::from_u32(options),
+        set,
+        payload,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
 fn waitable_set_poll(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller: u32,
     options: u32,
     set: u32,
     payload: u32,
 ) -> Result<u32> {
-    instance.waitable_set_poll(store, OptionsIndex::from_u32(options), set, payload)
+    instance.waitable_set_poll(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller),
+        OptionsIndex::from_u32(options),
+        set,
+        payload,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
@@ -777,8 +802,17 @@ fn waitable_join(
 }
 
 #[cfg(feature = "component-model-async")]
-fn thread_yield(store: &mut dyn VMStore, instance: Instance, cancellable: u8) -> Result<bool> {
-    instance.thread_yield(store, cancellable != 0)
+fn thread_yield(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    cancellable: u8,
+) -> Result<bool> {
+    instance.thread_yield(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        cancellable != 0,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
@@ -946,17 +980,23 @@ unsafe impl HostResultHasUnwindSentinel for ResourcePair {
 }
 
 #[cfg(feature = "component-model-async")]
-fn future_new(store: &mut dyn VMStore, instance: Instance, ty: u32) -> Result<ResourcePair> {
-    instance
-        .id()
-        .get_mut(store)
-        .future_new(TypeFutureTableIndex::from_u32(ty))
+fn future_new(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    ty: u32,
+) -> Result<ResourcePair> {
+    instance.id().get_mut(store).future_new(
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        TypeFutureTableIndex::from_u32(ty),
+    )
 }
 
 #[cfg(feature = "component-model-async")]
 fn future_write(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     future: u32,
@@ -964,6 +1004,7 @@ fn future_write(
 ) -> Result<u32> {
     store.component_async_store().future_write(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeFutureTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         future,
@@ -975,6 +1016,7 @@ fn future_write(
 fn future_read(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     future: u32,
@@ -982,6 +1024,7 @@ fn future_read(
 ) -> Result<u32> {
     store.component_async_store().future_read(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeFutureTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         future,
@@ -993,12 +1036,14 @@ fn future_read(
 fn future_cancel_write(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     async_: u8,
     writer: u32,
 ) -> Result<u32> {
     instance.future_cancel_write(
         store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeFutureTableIndex::from_u32(ty),
         async_ != 0,
         writer,
@@ -1009,12 +1054,14 @@ fn future_cancel_write(
 fn future_cancel_read(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     async_: u8,
     reader: u32,
 ) -> Result<u32> {
     instance.future_cancel_read(
         store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeFutureTableIndex::from_u32(ty),
         async_ != 0,
         reader,
@@ -1025,11 +1072,13 @@ fn future_cancel_read(
 fn future_drop_writable(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     writer: u32,
 ) -> Result<()> {
     store.component_async_store().future_drop_writable(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeFutureTableIndex::from_u32(ty),
         writer,
     )
@@ -1039,24 +1088,36 @@ fn future_drop_writable(
 fn future_drop_readable(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     reader: u32,
 ) -> Result<()> {
-    instance.future_drop_readable(store, TypeFutureTableIndex::from_u32(ty), reader)
+    instance.future_drop_readable(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        TypeFutureTableIndex::from_u32(ty),
+        reader,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
-fn stream_new(store: &mut dyn VMStore, instance: Instance, ty: u32) -> Result<ResourcePair> {
-    instance
-        .id()
-        .get_mut(store)
-        .stream_new(TypeStreamTableIndex::from_u32(ty))
+fn stream_new(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    ty: u32,
+) -> Result<ResourcePair> {
+    instance.id().get_mut(store).stream_new(
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        TypeStreamTableIndex::from_u32(ty),
+    )
 }
 
 #[cfg(feature = "component-model-async")]
 fn stream_write(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     stream: u32,
@@ -1065,6 +1126,7 @@ fn stream_write(
 ) -> Result<u32> {
     store.component_async_store().stream_write(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         stream,
@@ -1077,6 +1139,7 @@ fn stream_write(
 fn stream_read(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     stream: u32,
@@ -1085,6 +1148,7 @@ fn stream_read(
 ) -> Result<u32> {
     store.component_async_store().stream_read(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         stream,
@@ -1097,12 +1161,14 @@ fn stream_read(
 fn stream_cancel_write(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     async_: u8,
     writer: u32,
 ) -> Result<u32> {
     instance.stream_cancel_write(
         store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         async_ != 0,
         writer,
@@ -1113,12 +1179,14 @@ fn stream_cancel_write(
 fn stream_cancel_read(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     async_: u8,
     reader: u32,
 ) -> Result<u32> {
     instance.stream_cancel_read(
         store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         async_ != 0,
         reader,
@@ -1129,11 +1197,13 @@ fn stream_cancel_read(
 fn stream_drop_writable(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     writer: u32,
 ) -> Result<()> {
     store.component_async_store().stream_drop_writable(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         writer,
     )
@@ -1143,16 +1213,23 @@ fn stream_drop_writable(
 fn stream_drop_readable(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     reader: u32,
 ) -> Result<()> {
-    instance.stream_drop_readable(store, TypeStreamTableIndex::from_u32(ty), reader)
+    instance.stream_drop_readable(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        TypeStreamTableIndex::from_u32(ty),
+        reader,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
 fn flat_stream_write(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     payload_size: u32,
@@ -1163,6 +1240,7 @@ fn flat_stream_write(
 ) -> Result<u32> {
     store.component_async_store().flat_stream_write(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         payload_size,
@@ -1177,6 +1255,7 @@ fn flat_stream_write(
 fn flat_stream_read(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     payload_size: u32,
@@ -1187,6 +1266,7 @@ fn flat_stream_read(
 ) -> Result<u32> {
     store.component_async_store().flat_stream_read(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeStreamTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         payload_size,
@@ -1201,6 +1281,7 @@ fn flat_stream_read(
 fn error_context_new(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     debug_msg_address: u32,
@@ -1208,6 +1289,7 @@ fn error_context_new(
 ) -> Result<u32> {
     instance.error_context_new(
         store.store_opaque_mut(),
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeComponentLocalErrorContextTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         debug_msg_address,
@@ -1219,6 +1301,7 @@ fn error_context_new(
 fn error_context_debug_message(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     options: u32,
     err_ctx_handle: u32,
@@ -1226,6 +1309,7 @@ fn error_context_debug_message(
 ) -> Result<()> {
     store.component_async_store().error_context_debug_message(
         instance,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeComponentLocalErrorContextTableIndex::from_u32(ty),
         OptionsIndex::from_u32(options),
         err_ctx_handle,
@@ -1237,21 +1321,43 @@ fn error_context_debug_message(
 fn error_context_drop(
     store: &mut dyn VMStore,
     instance: Instance,
+    caller_instance: u32,
     ty: u32,
     err_ctx_handle: u32,
 ) -> Result<()> {
     instance.id().get_mut(store).error_context_drop(
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
         TypeComponentLocalErrorContextTableIndex::from_u32(ty),
         err_ctx_handle,
     )
 }
 
 #[cfg(feature = "component-model-async")]
-fn context_get(store: &mut dyn VMStore, instance: Instance, slot: u32) -> Result<u32> {
-    instance.concurrent_state_mut(store).context_get(slot)
+fn context_get(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    slot: u32,
+) -> Result<u32> {
+    instance.context_get(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        slot,
+    )
 }
 
 #[cfg(feature = "component-model-async")]
-fn context_set(store: &mut dyn VMStore, instance: Instance, slot: u32, val: u32) -> Result<()> {
-    instance.concurrent_state_mut(store).context_set(slot, val)
+fn context_set(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    slot: u32,
+    val: u32,
+) -> Result<()> {
+    instance.context_set(
+        store,
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        slot,
+        val,
+    )
 }
