@@ -95,11 +95,11 @@ mod test_vmfunction_import {
     use super::VMFunctionImport;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmfunction_import_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMFunctionImport>(),
@@ -165,11 +165,11 @@ mod test_vmtable {
     use core::mem::offset_of;
     use std::mem::size_of;
     use wasmtime_environ::component::{Component, VMComponentOffsets};
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmtable_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMTableImport>(),
@@ -194,7 +194,7 @@ mod test_vmtable {
         // Because we use `VMTableImport` for recording tables used by components, we
         // want to make sure that the size calculations between `VMOffsets` and
         // `VMComponentOffsets` stay the same.
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let vm_offsets = VMOffsets::new(HostPtr, &module);
         let component = Component::default();
         let vm_component_offsets = VMComponentOffsets::new(HostPtr, &component);
@@ -228,11 +228,11 @@ mod test_vmmemory_import {
     use super::VMMemoryImport;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmmemory_import_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMMemoryImport>(),
@@ -302,11 +302,11 @@ mod test_vmglobal_import {
     use super::VMGlobalImport;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmglobal_import_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMGlobalImport>(),
@@ -341,11 +341,11 @@ unsafe impl VmSafe for VMTagImport {}
 mod test_vmtag_import {
     use super::VMTagImport;
     use core::mem::{offset_of, size_of};
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmtag_import_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMTagImport>(),
@@ -418,11 +418,11 @@ mod test_vmmemory_definition {
     use super::VMMemoryDefinition;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, PtrSize, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, PtrSize, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmmemory_definition_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMMemoryDefinition>(),
@@ -465,11 +465,11 @@ mod test_vmtable_definition {
     use super::VMTableDefinition;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmtable_definition_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMTableDefinition>(),
@@ -504,7 +504,7 @@ unsafe impl VmSafe for VMGlobalDefinition {}
 mod test_vmglobal_definition {
     use super::VMGlobalDefinition;
     use std::mem::{align_of, size_of};
-    use wasmtime_environ::{HostPtr, Module, PtrSize, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, PtrSize, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmglobal_definition_alignment() {
@@ -519,7 +519,7 @@ mod test_vmglobal_definition {
 
     #[test]
     fn check_vmglobal_definition_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMGlobalDefinition>(),
@@ -529,7 +529,7 @@ mod test_vmglobal_definition {
 
     #[test]
     fn check_vmglobal_begins_aligned() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(offsets.vmctx_globals_begin() % 16, 0);
     }
@@ -777,11 +777,11 @@ impl VMGlobalDefinition {
 mod test_vmshared_type_index {
     use super::VMSharedTypeIndex;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmshared_type_index() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMSharedTypeIndex>(),
@@ -813,11 +813,11 @@ unsafe impl VmSafe for VMTagDefinition {}
 mod test_vmtag_definition {
     use super::VMTagDefinition;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, PtrSize, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, PtrSize, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vmtag_definition_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMTagDefinition>(),
@@ -827,7 +827,7 @@ mod test_vmtag_definition {
 
     #[test]
     fn check_vmtag_begins_aligned() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(offsets.vmctx_tags_begin() % 16, 0);
     }
@@ -973,11 +973,11 @@ mod test_vm_func_ref {
     use super::VMFuncRef;
     use core::mem::offset_of;
     use std::mem::size_of;
-    use wasmtime_environ::{HostPtr, Module, PtrSize, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, PtrSize, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn check_vm_func_ref_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             size_of::<VMFuncRef>(),
@@ -1141,22 +1141,34 @@ pub struct VMStoreContext {
 
     /// The last host stack pointer before we called into Wasm from the host.
     ///
-    /// Maintained by our host-to-Wasm trampoline, and cleared just before
-    /// calling into Wasm in `catch_traps`.
-    ///
-    /// This member is `0` when Wasm is actively running and has not called out
-    /// to the host.
+    /// Maintained by our host-to-Wasm trampoline. This member is `0` when Wasm
+    /// is not running, and it's set to nonzero once a host-to-wasm trampoline
+    /// is executed.
     ///
     /// When a host function is wrapped into a `wasmtime::Func`, and is then
-    /// called from the host, then this member has the sentinel value of `-1 as
-    /// usize`, meaning that this contiguous sequence of Wasm frames is the
-    /// empty sequence, and it is not safe to dereference the
-    /// `last_wasm_exit_trampoline_fp`.
+    /// called from the host, then this member is not changed meaning that the
+    /// previous activation in pointed to by `last_wasm_exit_trampoline_fp` is
+    /// still the last wasm set of frames on the stack.
     ///
-    /// Used to find the end of a contiguous sequence of Wasm frames when
-    /// walking the stack.
+    /// This field is saved/restored during fiber suspension/resumption
+    /// resumption as part of `CallThreadState::swap`.
+    ///
+    /// This field is used to find the end of a contiguous sequence of Wasm
+    /// frames when walking the stack. Additionally it's used when a trap is
+    /// raised as part of the set of parameters used to resume in the entry
+    /// trampoline's "catch" block.
+    pub last_wasm_entry_sp: UnsafeCell<usize>,
+
+    /// Same as `last_wasm_entry_sp`, but for the `fp` of the trampoline.
     pub last_wasm_entry_fp: UnsafeCell<usize>,
 
+    /// The last trap handler from a host-to-wasm entry trampoline on the stack.
+    ///
+    /// This field is configured when the host calls into wasm by the trampoline
+    /// itself. It stores the `pc` of an exception handler suitable to handle
+    /// all traps (or uncaught exceptions).
+    pub last_wasm_entry_trap_handler: UnsafeCell<usize>,
+
     /// Stack information used by stack switching instructions. See documentation
     /// on `VMStackChain` for details.
     pub stack_chain: UnsafeCell<VMStackChain>,
@@ -1250,6 +1262,8 @@ impl Default for VMStoreContext {
             last_wasm_exit_trampoline_fp: UnsafeCell::new(0),
             last_wasm_exit_pc: UnsafeCell::new(0),
             last_wasm_entry_fp: UnsafeCell::new(0),
+            last_wasm_entry_sp: UnsafeCell::new(0),
+            last_wasm_entry_trap_handler: UnsafeCell::new(0),
             stack_chain: UnsafeCell::new(VMStackChain::Absent),
             async_guard_range: ptr::null_mut()..ptr::null_mut(),
         }
@@ -1260,11 +1274,11 @@ impl Default for VMStoreContext {
 mod test_vmstore_context {
     use super::{VMMemoryDefinition, VMStoreContext};
     use core::mem::offset_of;
-    use wasmtime_environ::{HostPtr, Module, PtrSize, VMOffsets};
+    use wasmtime_environ::{HostPtr, Module, PtrSize, StaticModuleIndex, VMOffsets};
 
     #[test]
     fn field_offsets() {
-        let module = Module::new();
+        let module = Module::new(StaticModuleIndex::from_u32(0));
         let offsets = VMOffsets::new(HostPtr, &module);
         assert_eq!(
             offset_of!(VMStoreContext, stack_limit),
@@ -1302,6 +1316,14 @@ mod test_vmstore_context {
             offset_of!(VMStoreContext, last_wasm_entry_fp),
             usize::from(offsets.ptr.vmstore_context_last_wasm_entry_fp())
         );
+        assert_eq!(
+            offset_of!(VMStoreContext, last_wasm_entry_sp),
+            usize::from(offsets.ptr.vmstore_context_last_wasm_entry_sp())
+        );
+        assert_eq!(
+            offset_of!(VMStoreContext, last_wasm_entry_trap_handler),
+            usize::from(offsets.ptr.vmstore_context_last_wasm_entry_trap_handler())
+        );
         assert_eq!(
             offset_of!(VMStoreContext, stack_chain),
             usize::from(offsets.ptr.vmstore_context_stack_chain())
