@@ -203,7 +203,7 @@ struct InlinerFrame<'a> {
     memories: PrimaryMap<MemoryIndex, dfg::CoreExport<EntityIndex>>,
     tables: PrimaryMap<TableIndex, dfg::CoreExport<EntityIndex>>,
     globals: PrimaryMap<GlobalIndex, dfg::CoreExport<EntityIndex>>,
-    tags: PrimaryMap<GlobalIndex, dfg::CoreExport<EntityIndex>>,
+    tags: PrimaryMap<TagIndex, dfg::CoreExport<EntityIndex>>,
     modules: PrimaryMap<ModuleIndex, ModuleDef<'a>>,
 
     // component model index spaces
@@ -650,26 +650,35 @@ impl<'a> Inliner<'a> {
             // information and then new entries for each intrinsic are recorded.
             ResourceNew(id, ty) => {
                 let id = types.resource_id(id.resource());
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*ty, dfg::Trampoline::ResourceNew(id)));
+                let index = self.result.trampolines.push((
+                    *ty,
+                    dfg::Trampoline::ResourceNew {
+                        instance: frame.instance,
+                        ty: id,
+                    },
+                ));
                 frame.funcs.push((*ty, dfg::CoreDef::Trampoline(index)));
             }
             ResourceRep(id, ty) => {
                 let id = types.resource_id(id.resource());
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*ty, dfg::Trampoline::ResourceRep(id)));
+                let index = self.result.trampolines.push((
+                    *ty,
+                    dfg::Trampoline::ResourceRep {
+                        instance: frame.instance,
+                        ty: id,
+                    },
+                ));
                 frame.funcs.push((*ty, dfg::CoreDef::Trampoline(index)));
             }
             ResourceDrop(id, ty) => {
                 let id = types.resource_id(id.resource());
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*ty, dfg::Trampoline::ResourceDrop(id)));
+                let index = self.result.trampolines.push((
+                    *ty,
+                    dfg::Trampoline::ResourceDrop {
+                        instance: frame.instance,
+                        ty: id,
+                    },
+                ));
                 frame.funcs.push((*ty, dfg::CoreDef::Trampoline(index)));
             }
             BackpressureSet { func } => {
@@ -708,10 +717,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::TaskReturn { results, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::TaskReturn {
+                        instance: frame.instance,
+                        results,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             TaskCancel { func } => {
@@ -736,20 +749,26 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::WaitableSetWait { options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::WaitableSetWait {
+                        instance: frame.instance,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             WaitableSetPoll { options } => {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::WaitableSetPoll { options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::WaitableSetPoll {
+                        instance: frame.instance,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             WaitableSetDrop { func } => {
@@ -774,6 +793,7 @@ impl<'a> Inliner<'a> {
                 let index = self.result.trampolines.push((
                     *func,
                     dfg::Trampoline::ThreadYield {
+                        instance: frame.instance,
                         cancellable: *cancellable,
                     },
                 ));
@@ -804,10 +824,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::StreamNew { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::StreamNew {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             StreamRead { ty, options } => {
@@ -819,10 +842,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::StreamRead { ty, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::StreamRead {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             StreamWrite { ty, options } => {
@@ -834,10 +861,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::StreamWrite { ty, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::StreamWrite {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             StreamCancelRead { ty, func, async_ } => {
@@ -849,6 +880,7 @@ impl<'a> Inliner<'a> {
                 let index = self.result.trampolines.push((
                     *func,
                     dfg::Trampoline::StreamCancelRead {
+                        instance: frame.instance,
                         ty,
                         async_: *async_,
                     },
@@ -864,6 +896,7 @@ impl<'a> Inliner<'a> {
                 let index = self.result.trampolines.push((
                     *func,
                     dfg::Trampoline::StreamCancelWrite {
+                        instance: frame.instance,
                         ty,
                         async_: *async_,
                     },
@@ -876,10 +909,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::StreamDropReadable { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::StreamDropReadable {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             StreamDropWritable { ty, func } => {
@@ -888,10 +924,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::StreamDropWritable { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::StreamDropWritable {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             FutureNew { ty, func } => {
@@ -900,10 +939,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::FutureNew { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::FutureNew {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             FutureRead { ty, options } => {
@@ -915,10 +957,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::FutureRead { ty, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::FutureRead {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             FutureWrite { ty, options } => {
@@ -930,10 +976,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::FutureWrite { ty, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::FutureWrite {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             FutureCancelRead { ty, func, async_ } => {
@@ -945,6 +995,7 @@ impl<'a> Inliner<'a> {
                 let index = self.result.trampolines.push((
                     *func,
                     dfg::Trampoline::FutureCancelRead {
+                        instance: frame.instance,
                         ty,
                         async_: *async_,
                     },
@@ -960,6 +1011,7 @@ impl<'a> Inliner<'a> {
                 let index = self.result.trampolines.push((
                     *func,
                     dfg::Trampoline::FutureCancelWrite {
+                        instance: frame.instance,
                         ty,
                         async_: *async_,
                     },
@@ -972,10 +1024,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::FutureDropReadable { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::FutureDropReadable {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             FutureDropWritable { ty, func } => {
@@ -984,10 +1039,13 @@ impl<'a> Inliner<'a> {
                 else {
                     unreachable!()
                 };
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::FutureDropWritable { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::FutureDropWritable {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             ErrorContextNew { options } => {
@@ -995,10 +1053,14 @@ impl<'a> Inliner<'a> {
                 let func = options.core_type;
                 let options = self.adapter_options(frame, types, options);
                 let options = self.canonical_options(options);
-                let index = self
-                    .result
-                    .trampolines
-                    .push((func, dfg::Trampoline::ErrorContextNew { ty, options }));
+                let index = self.result.trampolines.push((
+                    func,
+                    dfg::Trampoline::ErrorContextNew {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
+                ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             ErrorContextDebugMessage { options } => {
@@ -1008,30 +1070,43 @@ impl<'a> Inliner<'a> {
                 let options = self.canonical_options(options);
                 let index = self.result.trampolines.push((
                     func,
-                    dfg::Trampoline::ErrorContextDebugMessage { ty, options },
+                    dfg::Trampoline::ErrorContextDebugMessage {
+                        instance: frame.instance,
+                        ty,
+                        options,
+                    },
                 ));
                 frame.funcs.push((func, dfg::CoreDef::Trampoline(index)));
             }
             ErrorContextDrop { func } => {
                 let ty = types.error_context_table_type()?;
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::ErrorContextDrop { ty }));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::ErrorContextDrop {
+                        instance: frame.instance,
+                        ty,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             ContextGet { func, i } => {
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::ContextGet(*i)));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::ContextGet {
+                        instance: frame.instance,
+                        slot: *i,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
             ContextSet { func, i } => {
-                let index = self
-                    .result
-                    .trampolines
-                    .push((*func, dfg::Trampoline::ContextSet(*i)));
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::ContextSet {
+                        instance: frame.instance,
+                        slot: *i,
+                    },
+                ));
                 frame.funcs.push((*func, dfg::CoreDef::Trampoline(index)));
             }
 
@@ -1357,7 +1432,7 @@ impl<'a> Inliner<'a> {
                 EntityIndex::Table(i) => frame.tables[i].clone().into(),
                 EntityIndex::Global(i) => frame.globals[i].clone().into(),
                 EntityIndex::Memory(i) => frame.memories[i].clone().into(),
-                EntityIndex::Tag(_) => todo!(), // FIXME: #10252 support for tags in the component model
+                EntityIndex::Tag(i) => frame.tags[i].clone().into(),
             },
         }
     }
